/*   
     
Chess-Ec    Copyright (C)   2001-2008 Marc Dechico


 ec_hash.c: Memorize and search movements allready simlated.
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 To contact the author:	Marc Dechico
 Kerfring
 29390 SCAER France
 tel: 06 37 98 22 54
 Mail: ec@chess-ec.org  
    
*/
/* target of the table:
   memorizing three subsequent movement and the note get by the computer
   after evaluation of the situation resulting from these three movements

   After three movement( depth of three ) the table will be searched to verify 
   if a similar situation has already been memorized:
   if not, the exploration will continue for the next level and the note will
   be memorized.
   if a similar situation is found, exploration is avoided by using the note already
   computed. 
   

   structure of the table:
   the arriving part of the movement is used as a hash-index. 
   for exemple:
   searching for  E2E4 E7E5 D2D3 (<=> D2d3 E7E5 E2E4)
   E4(field number 13)  is used as an index (struct st_arr)
   from the adresse found in st_arr I'll search in struct st_dep for E2 
   if found, I'll search again in st_dep for E5 and so on.
   if complet match is found, I get the note and go to the next move at same depth
   instead of diving further in the tree.

   the memory allocated is roughly 
   446 Ko for 40 mvt by level
   837 Ko for 55 mvt by level
   1   Mo for 60 mvt by level 
bug:
     C5F2,E2F2,H4F2 (all taken)  isn't equivalent to H4F2,E2F2,C5F2 
     this bug is corrected in ec_simu.c function  sim_add_l3()


*/
#include "ec_hash.h"

/*******************************************************************************************/
#ifdef TEST_HASH
/* these function are for test and debug purpose */
int ex_app()
{
  int i_not;
  int i_typ_pri_lea;
  struct st_hash *pst_hash;
  printf("\ntest hash");
      ini_mem(& pst_hash,3,50);
      print_st_hash(pst_hash);
 
 /*creation de la base */
      add_mvt_l1(pst_hash,E2,E4,' ');

      add_mvt_l2(pst_hash,E7,E5,' ');
      
      add_mvt_l3(pst_hash,F1,D3,' ');
      add_mvt_l3(pst_hash,G1,F3,' ');
      add_mvt_l3(pst_hash,D2,D3,' ');
      
      add_mvt_l2(pst_hash,D7,D5,' ');
      
      add_mvt_l3(pst_hash,D2,D4,' ');
      add_mvt_l3(pst_hash,D2,D3,' ');
      add_mvt_l3(pst_hash,F1,D3,' ');

      add_mvt_l1(pst_hash,E2,E3,' ');

      add_mvt_l2(pst_hash,E7,E5,' ');
      print_arr();print_dep();

 

/* interrogation de la base */	    
      if (rec_mvt_l3(pst_hash,E2,E4,' ',E7,E5,' ',F1,D3,' ',&i_not,&i_typ_pri_lea))
	  printf("\nfound E2E4 E7E5 F1D3");
      else
	  printf("\nnot found E2E4 E7E5 F1D3");

      if (rec_mvt_l3(pst_hash,E2,E4,' ',E7,E5,' ',G1,F3,' ',&i_not,&i_typ_pri_lea))
	  printf("\nfound E2E4 E7E5 G1F3");
      else
	  printf("\nnot found E2E4 E7E5 G1F3");

      if (rec_mvt_l3(pst_hash,E2,E4,' ',D7,D5,' ',D2,D3,' ',&i_not,&i_typ_pri_lea))
	  printf("\nfound E2E4 D7D5 D2D3");
      else
	  printf("\nnot found E2E4 D7D5 D2D3");

      if (rec_mvt_l3(pst_hash,E2,E4,' ',D7,D5,' ',F1,C4,' ',&i_not,&i_typ_pri_lea))
	  printf("\nfound E2E4 D7D5 F1C4");
      else
	printf("\nnot found E2E4 D7D5 F1C4");

      if (rec_mvt_l3(pst_hash,E2,E3,' ',E7,E5,' ',F1,C4,' ',&i_not,&i_typ_pri_lea))
	  printf("\nfound E2E3 E7E5 F1C4");
      else
	  printf("\nnot found E2E3 E7E5 F1C4");

      fre_mem(& pst_hash);
return 0;
}
void print_arr ()
{
  int i,j;
  struct st_arr *pst_arr;
  j=0;
  pst_arr=pst_hash->pst_arr_beg;
  while (pst_arr <pst_hash->pst_arr_end)
    {
      printf("\naddress of cur arr %li",(long)pst_arr);
      for (i=0;i<64;i++)
	{
	  if (pst_arr->pst_dep[i]!=NULL)
	    printf("\n\tSegment arr %i Poste %s addresse %li",j,tc_lib_cas[i+11],(long)pst_arr->pst_dep[i]); 
	}
      j++;
      pst_arr++;
    }
}  
void print_dep ()
{
  int i=0;
  struct st_dep *pst_dep;

  pst_dep=pst_hash->pst_dep_beg;
  while (pst_dep <pst_hash->pst_dep_end)
    {
      //printf("\naddress of cur dep %li",pst_dep);
	  if (pst_dep->i_dep!=0)
	    printf("\nSegment dep:%i  cur_add:%li  dep:%s adresse suivante %li addr niv sui %li ",
		   i,(long)pst_dep,tc_lib_cas[pst_dep->i_dep],(long)pst_dep->pst_nxt_dep,(long)pst_dep->pst_arr_nxt_lev); 

	  pst_dep++;i++;
    }
}  
void print_st_hash (struct st_hash *pst_hash)
{
  
  printf ("\nnbr mvt tot evalué %li",(long)pst_hash->li_mvt_qua_tot );
  printf ("\nAdr cur seg l1 dep  %li",(long)pst_hash->pst_dep_cur_l1);
  printf("\ncpomteur branche cherché:%li trouvé:%li",pst_hash->l_nbr_bra_sek,pst_hash->l_nbr_bra_fou);


  printf ("\n__________arr_______________________________");
  
  printf ("\nSize of struc arr %i",sizeof(struct st_arr) );
  printf ("\nSize of bloc arr %li",pst_hash->l_arr_siz );
  printf ("\nAdr Deb of bloc arr  %li",(long)pst_hash->pst_arr_beg);
  printf ("\nAdr End  of bloc arr  %li",(long)pst_hash->pst_arr_end);
  printf ("\nAdr free seg   arr  %li",(long)pst_hash->pst_arr_fre);
  printf ("\nnbr increm  arr  %i",pst_hash->i_arr_inc_nbr);
  printf ("\nnbr segment arr  %li",pst_hash->l_arr_seg_nbr);
  
  printf("\n___________dep_______________________________");

  printf ("\nSize of struc dep %i",sizeof(struct st_dep) );
  printf ("\nSize of bloc dep %li",(long)pst_hash->l_dep_siz );
  printf ("\nAdr Deb of bloc dep  %li",(long)pst_hash->pst_dep_beg);
  printf ("\nAdr End  of bloc dep  %li",(long)pst_hash->pst_dep_end);
  printf ("\nAdr free seg   dep  %li",(long)pst_hash->pst_dep_fre);
  printf ("\nnbr increm  dep  %i",pst_hash->i_dep_inc_nbr);
  printf ("\nnbr segment dep  %li",pst_hash->l_dep_seg_nbr);
  

  printf("\n===================================================\n");

}
#endif

int ini_mem (struct st_hash ** ppst_hash,int i_mvt_qua_ave,int i_inc_per)
{

  *ppst_hash=mem_malloc(sizeof(struct st_hash));
  memset(*ppst_hash,0,sizeof(struct st_hash));

  /*size default */ 
  if (i_mvt_qua_ave <=19) i_mvt_qua_ave=19;
  (*ppst_hash)->i_mvt_qua_ave=i_mvt_qua_ave;
  /* increment default */
  if (i_inc_per == 0) i_inc_per=30;
  (*ppst_hash)->i_inc_per=i_inc_per;

  /*deprecated */
   (*ppst_hash)->li_mvt_qua_tot=1 + i_mvt_qua_ave + i_mvt_qua_ave * i_mvt_qua_ave;

  /* size of the bloc depart  */ 
  /* no proof that the calculation is right under the  limit 15 */
  (*ppst_hash)->li_dep_mov_qua= ( (*ppst_hash)->i_mvt_qua_ave * 1.84 -26.8)*1000;

  /* size of the bloc  arriving */ 
  (*ppst_hash)->li_arr_mov_qua= (*ppst_hash)->li_dep_mov_qua  /
    ((*ppst_hash)->i_mvt_qua_ave * 0.5 + 4);

  /* initialize the memory list */
  mem_lst_ini(&((*ppst_hash)->pst_ind));

  /* alloc memory for the two tables 
     i_mvt_qua_ave is the average number of possible movements
     at each level (average is around 35)
     i_inc_per isthe amount in percentage of memory to add when totally used 
  */
  ini_mem_arr(*ppst_hash);
  ini_mem_dep(*ppst_hash);
  return 0;

}
int fre_mem(struct st_hash ** ppst_hash)
{
  extern int mem_lst_lst (struct st_ind *pst_cur_ind);
 
  // debug   mem_lst_lst(((*ppst_hash)->pst_ind));
  //print_st_hash(*ppst_hash);
  mem_lst_fre(&((*ppst_hash)->pst_ind));
  free(*ppst_hash);
  return 0;  
}
int ini_mem_arr(struct st_hash * pst_hash)
{
  /* initialise memory for the index (arrinving) of  he hash table */

  /* memorize quantitiy of memory used */
  pst_hash->l_arr_siz =sizeof(struct st_arr) *  pst_hash->li_arr_mov_qua;

  /* no increment yet */
  pst_hash->i_arr_inc_nbr=0;
 
  /* alloc memory */
  pst_hash->pst_arr_fre=mem_malloc(pst_hash->l_arr_siz); 

  /* memorise adresse of the index-root for the root of mvt tree */
  pst_hash->pst_arr_beg=pst_hash->pst_arr_fre;

  /* memorise end-adresse of memory block*/
  pst_hash->pst_arr_end=pst_hash->pst_arr_fre -2 + pst_hash->li_arr_mov_qua;
 
  /* initialize memory */ 
  memset(pst_hash->pst_arr_fre,0,pst_hash->l_arr_siz);

  /* add this memory-block to the garbage collector */
  mem_lst_add(&pst_hash->pst_ind,(void *) pst_hash->pst_arr_fre);
  /* debug */
  //printf("\n init memory  arr pst_ind=%li pre_ind=%li",(long)pst_hash->pst_ind,(long)pst_hash->pst_ind->pst_pre_ind);
  return 0;
}

int ini_mem_dep(struct st_hash * pst_hash)
{
   /* initialise memory for datas (departure...) of  he hash table */

  /* memorize quantitiy of memory used */
  pst_hash->l_dep_siz =sizeof(struct st_dep) *  pst_hash->li_dep_mov_qua ;

  /* no increment yet */
  pst_hash->i_dep_inc_nbr=0;
 
  /* alloc memory */
  pst_hash->pst_dep_fre=mem_malloc(pst_hash->l_dep_siz); 
  pst_hash->pst_dep_beg= pst_hash->pst_dep_fre;
 
  /* memorise end-adresse of memory block*/
  pst_hash->pst_dep_end=pst_hash->pst_dep_fre -2  + pst_hash->li_dep_mov_qua;

  /* initialize memory */ 
  memset(pst_hash->pst_dep_fre,0,pst_hash->l_dep_siz);

  /* add this memory-block to the garbage collector */
  mem_lst_add(&pst_hash->pst_ind,(void *)pst_hash->pst_dep_fre);
  /* debug */
  //printf("\n init memory  dep pst_ind=%li pre_ind=%li",(long)pst_hash->pst_ind,(long)pst_hash->pst_ind->pst_pre_ind);
  return 0;
}


int inc_mem_arr(struct st_hash * pst_hash)
{
  //printf("\n Incremente bloc arr"); 
  /* increment  memory for the index (arriving) of  he hash table */

  
  /* quantitiy of mouvement for  increment */
  long l_nbr_mvt_inc   =  pst_hash->li_arr_mov_qua  * pst_hash->i_inc_per / 100.00;
  long l_arr_siz_inc  = sizeof(struct st_arr) * l_nbr_mvt_inc;
  
    pst_hash->l_arr_siz+=l_arr_siz_inc;

  /* memorize how many increment  */
  pst_hash->i_arr_inc_nbr++;
 
  /* alloc memory */
  pst_hash->pst_arr_fre=mem_malloc(l_arr_siz_inc); 

  /* memorise end-adresse of memory block*/
  pst_hash->pst_arr_end=pst_hash->pst_arr_fre -2 +l_nbr_mvt_inc ;
 
  /* initialize memory */ 
  memset(pst_hash->pst_arr_fre,0,l_arr_siz_inc);

  /* add this memory-block to the garbage collector */
  mem_lst_add(&pst_hash->pst_ind,(void *)pst_hash->pst_arr_fre);

  /* debug */
  //  printf("\nadd increment  arr pst_ind=%li pre_ind=%li",(long)pst_hash->pst_ind,(long)pst_hash->pst_ind->pst_pre_ind);
  return 0;

}
int inc_mem_dep(struct st_hash * pst_hash)
{
  // printf("\n Incremente bloc dep"); 
  /* increment  memory for the index (arriving) of  he hash table */

  
  /* quantitiy of mouvement for  increment */
  long l_nbr_mvt_inc   =  pst_hash->li_dep_mov_qua  * pst_hash->i_inc_per / 100.00;
  long l_dep_siz_inc  = sizeof(struct st_dep) * l_nbr_mvt_inc;
  
    pst_hash->l_dep_siz+=l_dep_siz_inc;

  /* memorize how many increment  */
  pst_hash->i_dep_inc_nbr++;
 
  /* alloc memory */
  pst_hash->pst_dep_fre=mem_malloc(l_dep_siz_inc); 

  /* memorise end-adresse of memory block*/
  pst_hash->pst_dep_end=pst_hash->pst_dep_fre -2 +l_nbr_mvt_inc ;
 
  /* initialize memory */ 
  memset(pst_hash->pst_dep_fre,0,l_dep_siz_inc);

  /* add this memory-block to the garbage collector */
  mem_lst_add(&pst_hash->pst_ind,(void *)pst_hash->pst_dep_fre);

  /* debug */
  // printf("\nadd increment dep pst_ind=%li pre_ind=%li",(long)pst_hash->pst_ind,(long)pst_hash->pst_ind->pst_pre_ind);
  return 0;

}

int get_nxt_arr (struct st_hash *pst_hash) 
{
  pst_hash->l_arr_seg_nbr++;

  /* verify if more memory has to be allocated */
  if (pst_hash->pst_arr_end == pst_hash->pst_arr_fre)
    {
      inc_mem_arr(pst_hash);
    }
  else
    {
      /* increment next free segment pointer  for arr */
      pst_hash->pst_arr_fre ++;
    }

  return 0;
}
int get_nxt_dep (struct st_hash *pst_hash) 
{
  pst_hash->l_dep_seg_nbr++;
  /* verify if  more memory has to be allocated */
  if (pst_hash->pst_dep_end == pst_hash->pst_dep_fre)
    {
      inc_mem_dep(pst_hash);
    }
  else
    {
      /* increment next free segment pointer  for arr */
      pst_hash->pst_dep_fre ++;
    }
  return 0;
}


/* memorised a movement at level n:
   if there is no movement at next depth level memorised the note
   else memorised the adresse where the next-depth movement will be memorized
   two table beeing used for one movement, if two movement have the same 
   arriving  field they are "chained" first in last out   */

int add_mvt_l1(struct st_hash  *pst_hash ,int i_dep, int i_arr, int i_typ_mvt) 
{
  /* I verify if a move allready exist for the arriving square at top level in the ariving memmory bloc.
     If a move exists (pointer != NULL)
        then  I update the departure segment fot this move with the link(adress) on the new free segment 
	I will use (to be able to get all move arriving on this square).

        else I update the arriving segment  with a link(adress) on this same new free 	segment 

     I update the data in the again same new free segment in the departure bloc with the departure 
     square and eventually the type of pawn promotion.
     
     I store this again same new free segement (in the hash structure) to be alble to update it
     at next level with a link to the next subsequent move.
     
     I store (in the hash structure) the adresse of a new free segment of the arriving memory bloc 

  */
  
  struct st_dep * pointer;
 

  pointer=(pst_hash->pst_arr_beg)->pst_dep[i_arr-11];
     
  /* if a movement having same arriving field has already been added */ 
  if ( pointer != NULL)
    /* I memorise a pointer to  previous departure bloc in the new bloc  */
    pst_hash->pst_dep_fre->pst_nxt_dep=pointer;
  else
    /* there is not next bloc */
    pst_hash->pst_dep_fre->pst_nxt_dep=NULL;

  /* I memorize this new segment for the arriving  square */
  pst_hash->pst_arr_beg->pst_dep[i_arr-11]=pst_hash->pst_dep_fre;

  /* I memorise the number of the departure square and the type of promotion  */
  pst_hash->pst_dep_fre->i_dep=i_dep;
  pst_hash->pst_dep_fre->i_typ_mvt=i_typ_mvt;
  /* initialize the other datas */
  pst_hash->pst_dep_fre->pst_arr_nxt_lev=NULL;

  /* memorize the current move at level1 */
  pst_hash->pst_dep_cur_l1=pst_hash->pst_dep_fre;
  
  /* I get the next free departure segment and store it in st_hash */ 
  get_nxt_dep(pst_hash);

  return 0;  /* dive */
 
}

int add_mvt_l2(struct st_hash  *pst_hash ,int i_dep, int i_arr,int i_typ_mvt) 
{
  /* 
     if the move at level1 isn't linked to a move of level2 (fist move at present level)
     then 
     get a new segment in the memory bloc arr (arriving)
     update the link with current move at previous level
     create the first move at current level (link to curent dep segement)
     else
     if a move for this arriving square already exixts     
     then
     link this  new  move (dep segment)  to the previous at the same current level2

     else 
     link this new move directly to the arr segment for the new ariving square
           
	   
     update new move  datas in the new dep segment square: 
     pawn promotion in the new dep segment.
     initialize data pointer in the new departure segment.
     
     get the next free segment in the dep memory bloc 

  */
  struct st_dep * pointer;

  i_arr=i_arr-11;

  /* check if the current move at at previous level1 is linked with a move at level2*/
  if(pst_hash->pst_dep_cur_l1->pst_arr_nxt_lev==NULL)
    {
      /* This is the first move at current level*/
      /* get the next segment in the arr memory bloc*/
      get_nxt_arr(pst_hash);
      /* link previous level with this new move arr segment */
      pst_hash->pst_dep_cur_l1->pst_arr_nxt_lev=pst_hash->pst_arr_fre;
      /* link the new dep segment with the arr segment */
      pst_hash->pst_arr_fre->pst_dep[i_arr]=pst_hash->pst_dep_fre;
      /* there is no previous move */
      pst_hash->pst_dep_fre->pst_nxt_dep=NULL;
      
    }
  else 
    {
      /* check if a move for this departure square already exists */
      pointer=pst_hash->pst_dep_cur_l1->pst_arr_nxt_lev->pst_dep[i_arr];
      if(pointer!=NULL)
	/* link to the previous move at current level */
	pst_hash->pst_dep_fre->pst_nxt_dep=pointer;
      else 
	/* there is no previous move */
	pst_hash->pst_dep_fre->pst_nxt_dep=NULL;

      /* link directly to the arr segment */
      pst_hash->pst_dep_cur_l1->pst_arr_nxt_lev->pst_dep[i_arr]=pst_hash->pst_dep_fre;
    } 

  /* I memorise the number of the departure square and the type of promotion  */
  pst_hash->pst_dep_fre->i_dep=i_dep;
  pst_hash->pst_dep_fre->i_typ_mvt=i_typ_mvt;
  /* initialize the other datas */
  pst_hash->pst_dep_fre->pst_arr_nxt_lev=NULL;

   /* memorize the current move at level2 */
  pst_hash->pst_dep_cur_l2=pst_hash->pst_dep_fre;
  
  /* I get the next free departure segment and store it in st_hash */ 
  get_nxt_dep(pst_hash);

  return 0; /*dive */

}
int add_mvt_l3(struct st_hash  *pst_hash ,int i_dep, int i_arr, int i_typ_mvt) 
{
  /* 
     if the move at level2 isn't linked to a move of level3 (fist move at present level)
     then 
     get a new segment in the memory bloc arr (arriving)
     update the link with current move at previous level
     create the first move at current level (link to curent dep segement)
     else
     if a move for this arriving square already exixts     
     then
     link this  new  move (dep segment)  to the previous at the same current level3

     else 
     link this new move directly to the arr segment for the new ariving square
           
	   
     update new move  datas in the new dep segment square: 
     pawn promotion in the new dep segment.
     initialize data pointer in the new departure segment.
     
     get the next free segment in the dep memory bloc 

  */
  struct st_dep * pointer;

  i_arr=i_arr-11;

  /* check if the current move at at previous level2 is linked with a move at level3*/
  if(pst_hash->pst_dep_cur_l2->pst_arr_nxt_lev==NULL)
    {
      /* This is the first move at current level*/
      /* get the next segment in the arr memory bloc*/
      get_nxt_arr(pst_hash);
      /* link previous level with this new move arr segment */
      pst_hash->pst_dep_cur_l2->pst_arr_nxt_lev=pst_hash->pst_arr_fre;
      /* link the new dep segment with the new  arr segment */
      pst_hash->pst_arr_fre->pst_dep[i_arr]=pst_hash->pst_dep_fre;
      /* there is no previous move */
      pst_hash->pst_dep_fre->pst_nxt_dep=NULL;
      
    }
  else 
    {
      /* check if a move for this departure square already exists */
      pointer=pst_hash->pst_dep_cur_l2->pst_arr_nxt_lev->pst_dep[i_arr];
      if(pointer!=NULL)
	/* link to the previous move at current level */
	pst_hash->pst_dep_fre->pst_nxt_dep=pointer;
      else
	/* there is no previous move */
	pst_hash->pst_dep_fre->pst_nxt_dep=NULL;

      /* link directly to the arr segment */
      pst_hash->pst_dep_cur_l2->pst_arr_nxt_lev->pst_dep[i_arr]=pst_hash->pst_dep_fre;

    } 

  /* I memorise the number of the departure square and the type of promotion  */
  pst_hash->pst_dep_fre->i_dep=i_dep;
  pst_hash->pst_dep_fre->i_typ_mvt=i_typ_mvt;
  /* initialize the other datas */
  pst_hash->pst_dep_fre->pst_arr_nxt_lev=NULL;
   /* memorize the current move at level3 */
  pst_hash->pst_dep_cur_l3=pst_hash->pst_dep_fre;
  
  /* I get the next free departure segment and store it in st_hash */ 
  get_nxt_dep(pst_hash);

  return 0; /* dive */

}



int rec_mvt_l3 (struct st_hash *pst_hash,
		int i_dep1, int i_arr1, int i_typ_mvt1, 
		int i_dep2, int i_arr2, int i_typ_mvt2, 
		int i_dep3, int i_arr3, int i_typ_mvt3, 
		int *pi_not,
		int *pi_not_cost,
		int *pi_not_pos,
		int *pi_typ_pri_lea)
{
  /* 
    
  
  */
  struct st_arr *pst_arr;
  pst_arr= pst_hash->pst_arr_beg;
  pst_hash->l_nbr_bra_sek++;


  /* 
     C5F2,E2F2,H4F2 (all taken)  isn't equivalent to H4F2,E2F2,C5F2 
     this bug is corrected in ec_simu.c function  sim_add_l3()
  */
  

  /* search for the first movement */
  if (rec_one_mvt (&pst_arr, i_dep1, i_arr1, i_typ_mvt1))
    {
      //printf("\nfound first part");
      /* search for the second one */
      if (rec_one_mvt (&pst_arr, i_dep2, i_arr2, i_typ_mvt2))
	{
	  //printf("\nfound second part");
	  /* search for the third  one */
	  if (rec_one_mvt_l3 (&pst_arr, i_dep3, i_arr3, i_typ_mvt3,
			      &pi_not,
			      &pi_not_cost,
			      &pi_not_pos,
			      &pi_typ_pri_lea))
	    {
	      //printf("\nfound third part");
	      pst_hash->l_nbr_bra_fou++;
	      return 1;
	    }

	  else return 0;
	}
      else
	return 0;
    }
  else
    return 0;
  
  return 0;
}

int rec_one_mvt (struct st_arr ** pst_arr, int  i_dep, int i_arr, int i_typ_mvt)
{
  /* This function check the existance of one move independently of the level 
     the link between two subsequents moves is used as the primary key (pst_arr)

     the segment pointed by this primary key is in the arr bloc and  contain a table
     I check the element of this table  with the square number given in the argument i_arr.
     (this is the hash : several move can arrive on the same square ) 
     if the move exists, a link to a dep segment must exist (element is a pointer!= NULL)
     If  there is several move arriving on the same square then several dep segments
     are linked together in a liste.
     I have to check them all until I find the move and return data:
     either a link to the next moves (pst_arr) or a link the data collected.and the note (pi_not)

     Normally after three subsequent move we should usuallay get a "simetrical situation"
     E2E4-E7E5-D2D4 = D2D4-E7E5-E2E4.
     Normally we should get only one move arrving on the same square.
     Time should be divided by two even in the starting situation.
  */

  struct st_dep * pointer;
  /* I get the move from the hash table */
  pointer=(*pst_arr)->pst_dep[(i_arr)-11];

  //printf("\n rec 1 mvt %s  pst_dep %li",tc_lib_cas[i_arr],pointer);
  while (pointer!=NULL)
    {
      if (  (pointer->i_dep == i_dep) 
	    & 
	    (pointer->i_typ_mvt == i_typ_mvt)
	 )
	  
	{
	  if(pointer->pst_arr_nxt_lev!=NULL)
	    {
	      /* return the data found */
	      *pst_arr=pointer->pst_arr_nxt_lev;
	      return 1;
	    }
	  else 
	    return 0;
	}
      else
	{
	  /* get next segment */
	  pointer=pointer->pst_nxt_dep;
	}
    }
  /* after checking all move hasn't found a match */ 
  return 0 ; /* not found */
}

int rec_one_mvt_l3 (struct st_arr ** pst_arr, int  i_dep, int i_arr, int i_typ_mvt,
		    int **pi_not,
		    int **pi_not_cost,
		    int **pi_not_pos,
		    int **pi_typ_pri_lea)
{
  /* This function check the existance of one move independently of the level 
     the link between two subsequents moves is used as the primary key (pst_arr)

     the segment pointed by this primary key is in the arr bloc and  contain a table
     I check the element of this table  with the square number given in the argument i_arr.
     (this is the hash : several move can arrive on the same square ) 
     if the move exists, a link to a dep segment must exist (element is a pointer!= NULL)
     If  there is several move arriving on the same square then several dep segments
     are linked together in a liste.
     I have to check them all until I find the move and return data:
     either a link to the next moves (pst_arr) or a link the data collected.and the note (pi_not)

     Normally after three subsequent move we should usuallay get a "simetrical situation"
     E2E4-E7E5-D2D4 = D2D4-E7E5-E2E4.
     Normally we should get only one move arrving on the same square.
     Time should be divided by two even in the starting situation.
  */

  struct st_dep * pointer;
  /* I get the move from the hash table */
  pointer=(*pst_arr)->pst_dep[(i_arr)-11];

  //printf("\n rec 1 mvt %s  pst_dep %li",tc_lib_cas[i_arr],pointer);
  while (pointer!=NULL)
    {
      if (  (pointer->i_dep == i_dep) 
	    & 
	    (pointer->i_typ_mvt == i_typ_mvt)
         ) 
	{
	  /* return the data found */
	  **pi_not=pointer->i_not;
	  **pi_not_cost=pointer->i_not_cost;
	  **pi_not_pos=pointer->i_not_pos;
	  **pi_typ_pri_lea=pointer->i_typ_pri_lea;
	  *pst_arr=pointer->pst_arr_nxt_lev;
	  return 1;
	}
      else
	{
	  /* get next segment */
	  pointer=pointer->pst_nxt_dep;
	}
    }
  /* after checking all move hasn't found a match */ 
  return 0 ; /* not found */
}
